# ! Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from json                import loads
from Crypto.Hash         import MD5
from Crypto.Cipher       import AES
from Crypto.Util.Padding import unpad
from base64              import b64decode

def decrypt_aes_with_custom_kdf(crypted_data, password) -> str:
    """
    AES/CBC/PKCS5Padding şifreleme şemasını kullanarak şifre çözme işlemi yapar.

    :param crypted_data: JSON formatında şifrelenmiş veri (ct, iv, s içerir).
    :param password: Anahtar türetmede kullanılacak şifre.
    :return: Çözülmüş veri (string olarak).
    """

    def generate_key_and_iv(password, salt, key_length=32, iv_length=16, iterations=1):
        """Anahtar ve IV oluşturmak için bir KDF fonksiyonu."""
        d = d_i = b""
        while len(d) < key_length + iv_length:
            d_i = MD5.new(d_i + password + salt).digest()
            for _ in range(1, iterations):
                d_i = MD5.new(d_i).digest()
            d += d_i
        return d[:key_length], d[key_length : key_length + iv_length]

    def hex_to_bytes(hex_str):
        """Hex string'i byte array'e çevirir."""
        return bytes.fromhex(hex_str)

    data      = loads(crypted_data)

    key, iv   = generate_key_and_iv(password, hex_to_bytes(data["s"]), iv_length=len(data["iv"]) // 2)

    cipher    = AES.new(key, AES.MODE_CBC, iv)
    decrypted = unpad(cipher.decrypt(b64decode(data["ct"])), AES.block_size)

    return decrypted.decode("utf-8")


# ! Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

import re

def unpack_packer(source: str) -> str:
    """https://github.com/beautifier/js-beautify/blob/main/python/jsbeautifier/unpackers/packer.py"""

    def clean_escape_sequences(source: str) -> str:
        source = re.sub(r'\\\\', r'\\', source)
        source = source.replace("\\'", "'")
        source = source.replace('\\"', '"')
        return source

    source = clean_escape_sequences(source)

    def extract_arguments(source: str) -> tuple[str, list[str], int, int]:
        match = re.search(r"}\('(.*)',(\d+),(\d+),'(.*)'\.split\('\|'\)", source, re.DOTALL)

        if not match:
            raise ValueError("Invalid P.A.C.K.E.R. source format.")

        payload, radix, count, symtab = match.groups()

        return payload, symtab.split("|"), int(radix), int(count)

    def convert_base(s: str, base: int) -> int:
        alphabet = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

        return sum(alphabet.index(char) * (base**idx) for idx, char in enumerate(reversed(s)))

    payload, symtab, radix, count = extract_arguments(source)

    if count != len(symtab):
        raise ValueError("Malformed P.A.C.K.E.R. symtab.")

    def lookup_symbol(match: re.Match) -> str:
        word = match[0]

        return symtab[convert_base(word, radix)] or word

    unpacked_source = re.sub(r"\b\w+\b", lookup_symbol, payload)

    return unpacked_source


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from .AtobRtt  import atob, rtt
from .Aes      import decrypt_aes_with_custom_kdf
from .Packer   import unpack_packer
from .CryptoJS import CryptoJS


# ! Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

import base64

def atob(s:str) -> str:
    return base64.b64decode(s).decode("utf-8")

def rtt(s:str) -> str:
    def rot13_char(c):
        if "a" <= c <= "z":
            return chr((ord(c) - ord("a") + 13) % 26 + ord("a"))
        elif "A" <= c <= "Z":
            return chr((ord(c) - ord("A") + 13) % 26 + ord("A"))
        return c

    return "".join(rot13_char(c) for c in s)